import greenfoot.*;

/**
 * A policeman that can be controlled by the arrowkeys: up, left, right.
 * The gun is fired by hitting the 'space' key. 'z' releases a proton wave.
 * 
 * @author Poul Henriksen
 * @author Michael Kölling
 * 
 * @version 1.0
 */
public class Policeman extends SmoothMover
{
    private static final int gunReloadTime = 5;         // The minimum delay between firing the gun.
    private static final int protonReloadTime = 500;    // The minimum delay between proton wave bursts.

    private int reloadDelayCount;               // How long ago we fired the gun the last time.
    private int protonDelayCount;               // How long ago we fired the proton wave the last time.
    
    private GreenfootImage policeman = new GreenfootImage("policeman.png");    
    

    /**
     * Initialise this policeman.
     */
    public Policeman()
    {
        reloadDelayCount = 5;
        protonDelayCount = 500;
        addToVelocity(new Vector(13, 0.7));    // initially slowly drifting
        
        
    }

    /**
     * Do what a policeman's gotta do. (Which is: mostly flying about, and turning,
     * accelerating and shooting when the right keys are pressed.)
     */
    public void act()
    {
        move();
        checkKeys();
        checkCollision();
        reloadDelayCount++;
        protonDelayCount++;
    }
    
    /**
     * Check whether there are any key pressed and react to them.
     */
    private void checkKeys() //exercise 9.20 - the checkKeys() method handles keyboard input. This includes any movements that you control invloving the policeman.
    {
        ignite(Greenfoot.isKeyDown("up"));
        
        if (Greenfoot.isKeyDown("left")) //exercise 9.21 - this code is saying that when you press the left key on the keyboard, the policeman will turn left 5 degress in every act cycle.
        {
            turn(-5);
        }
        if (Greenfoot.isKeyDown("right")) //exercise 9.22 - The Rocket will turn right when you press the right arrow key on the keyboard.
        {
            turn(5);
        }
        if (Greenfoot.isKeyDown("space")) 
        {
            fire();
        }
        if (Greenfoot.isKeyDown("z")) 
        {
            startProtonWave();
        }
    }
    
    /**
     * Check whether we are colliding with an doughnut.
     */
    private void checkCollision() 
    {
        Doughnut a = (Doughnut) getOneIntersectingObject(Doughnut.class);
        if (a != null) 
        {
            Town space = (Town) getWorld();
            space.addObject(new Explosion(), getX(), getY());
            space.removeObject(this);
            space.gameOver();
        }
    }
    
    /**
     * Should the policeman be ignited?
     */
    private void ignite(boolean boosterOn) 
    {
        if (boosterOn) 
        {
            addToVelocity(new Vector(getRotation(), 0.3));
        }
        else 
        {
            setImage(policeman);        
        }
    }
    
    /**
     * Fire a projectile if the gun is ready.
     */
    private void fire() 
    {
        if (reloadDelayCount >= gunReloadTime) 
        {
            Projectile projectile = new Projectile (getVelocity(), getRotation());
            getWorld().addObject (projectile, getX(), getY());
            projectile.move ();
            reloadDelayCount = 0;
        }
    }
    
    /**
     * Release a proton wave (if it is loaded).
     */
    private void startProtonWave() 
    {
        if (protonDelayCount >= protonReloadTime) 
        {
            ProtonWave wave = new ProtonWave();
            getWorld().addObject (wave, getX(), getY());
            protonDelayCount = 0;
        }
    }

}